from ..lint.unix import parse_report, LintEntry

raw_report = """\
/var/lib/jenkins/workspace/Example project/components/DropdownMenu.jsx:6:1: Component should be written as a pure function [Error/react/prefer-stateless-function]
/var/lib/jenkins/workspace/Example project/components/DropdownMenu.jsx:26:3: Prop type `object` is forbidden [Error/react/forbid-prop-types]
/var/lib/jenkins/workspace/Example project/components/DropdownMenu.jsx:73:7: Do not use setState in componentDidUpdate [Error/react/no-did-update-set-state]
/var/lib/jenkins/workspace/Example project/components/DropdownMenu.jsx:112:28: Unexpected usage of singlequote. [Warning/jsx-quotes]
/var/lib/jenkins/workspace/Example project/index.js:2:8: 'jquery' is defined but never used. [Warning/no-unused-vars]
/var/lib/jenkins/workspace/Example project/package/utils.js:1:15: Strings must use singlequote. [Warning/quotes]
/var/lib/jenkins/workspace/Example project/package/utils.js:2:15: Strings must use singlequote. [Warning/quotes]
/var/lib/jenkins/workspace/Example project/package/utils.js:4:1: Prefer default export. [Warning/import/prefer-default-export]
/var/lib/jenkins/workspace/Example project/package/utils.js:4:14: Identifier 'package_utils' is not in camel case. [Warning/camelcase]
/var/lib/jenkins/workspace/Example project/package/utils.js:5:1: Expected indentation of 2 spaces but found 4. [Error/indent]
"""


def test_parse_report_parses_raw_report():
    report = parse_report(raw_report)
    assert len(report) == 10
    assert all((isinstance(item, LintEntry)) for item in report)



from __future__ import print_function, unicode_literals

from ..git import parse_hunk, parse_file_line_numbers


def test_parse_hunk():
    assert parse_hunk('@@ -0 +1 @@ Foo bar') == ((0, 1), (1, 1), u'Foo bar')
    assert parse_hunk('@@ -987 +99999 @@ Foo bar') == ((987, 1), (99999, 1), u'Foo bar')
    assert parse_hunk('@@ -5,0 +42,5 @@ Foo bar') == ((5, 0), (42, 5), u'Foo bar')
    assert parse_hunk('@@ -1,3 +42,0 @@ Foo bar') == ((1, 3), (42, 0), u'Foo bar')
    assert parse_hunk('@@ -0 +1 @@') == ((0, 1), (1, 1), u'')


def test_parse_file_line_numbers():
    assert parse_file_line_numbers('-0') == (0, 1)
    assert parse_file_line_numbers('+0') == (0, 1)
    assert parse_file_line_numbers('+0,0') == (0, 0)
    assert parse_file_line_numbers('+0,1') == (0, 1)
    assert parse_file_line_numbers('+0,5') == (0, 5)
    assert parse_file_line_numbers('+123,5') == (123, 5)



from __future__ import print_function, unicode_literals

import subprocess
import re

import six


def get_diff_lines(revision_range):
    diff_content = get_diff(revision_range)
    for entry in parse_diff(diff_content):
        yield entry


def get_diff(revision_range):
    cmd_output = subprocess.check_output(['git', 'diff', revision_range, '--unified=0'],
                                         universal_newlines=True)
    if isinstance(cmd_output, six.binary_type):
        cmd_output = cmd_output.decode('utf-8')
    return cmd_output


def parse_diff(diff_content):
    lines = diff_content.split('\n')

    a_filename, b_filename = None, None
    hunks = None

    for line in lines:
        if line.startswith('diff --git'):
            # Process next file
            if hunks:
                yield (a_filename, b_filename), hunks
            hunks = []

        elif line.startswith('--- '):
            a_filename = parse_diff_filename(line)

        elif line.startswith('+++ '):
            b_filename = parse_diff_filename(line)

        elif line.startswith('@@'):
            hunk = parse_hunk(line)
            hunks.append(hunk)

    # Last file
    if hunks:
        yield (a_filename, b_filename), hunks


def parse_diff_filename(line):
    if isinstance(line, six.binary_type):
        line = line.decode('unicode-escape')
    line_decoded = line.encode('latin-1').decode('utf-8')
    without_prefix = line_decoded[4:].rstrip()
    if without_prefix == '/dev/null':
        return ''
    without_quotes = without_prefix[1:-1] if without_prefix.startswith('"') else without_prefix
    filename = without_quotes[2:]  # Strip a/b
    return filename


def parse_hunk(line):
    """
    Parse git hunk (Example: "@@ -5,0 +42,5 @@ Foobar")
    """
    hunk_parts = re.match(r"^@@ ([^@ ]+) ([^@ ]+) @@ ?(.*)$", line)
    line_from_formatted, line_to_formatted, code = hunk_parts.groups()

    line_from = parse_file_line_numbers(line_from_formatted)
    line_to = parse_file_line_numbers(line_to_formatted)

    return line_from, line_to, code


def parse_file_line_numbers(formatted_numbers):
    """
    Parse "start,count" formatted line numbers
    """
    formatted = formatted_numbers[1:]  # strip -/+
    formatted_parts = formatted.split(',')
    start = int(formatted_parts[0])
    count = int(formatted_parts[1]) if len(formatted_parts) > 1 else 1
    return start, count



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals

import argparse
import codecs

import lintlens
from .git import get_diff_lines
from .lint.unix import parse_lint_line
from .utils import check_line_overlap_hunks


def handle_range(revision_range, lint_lines):
    diff_lines = {}
    for filename, hunks in get_diff_lines(revision_range):
        diff_lines[filename[1]] = hunks

    for lint_line in lint_lines:
        lint_entry = parse_lint_line(lint_line)

        # skip file not changed in revision_range
        if lint_entry.filename not in diff_lines:
            continue

        hunks = diff_lines[lint_entry.filename]
        if check_line_overlap_hunks(lint_entry.line, hunks, threshold=1):
            print(lint_line, end='')


def read_file_lines(filename):
    with codecs.open(filename, 'r', encoding='utf-8') as f:
        lines = f.readlines()
    return lines


def main():
    parser = argparse.ArgumentParser(prog='lintlens')
    parser.add_argument('--version', action='version', version='%(prog)s ' + str(lintlens.__version__))
    parser.add_argument('revision_range',
                        help='Include changes in the specified revision range. '
                             'Example: "master..HEAD".'
                        )
    parser.add_argument('input_filename',
                        help='Input filename')

    args = parser.parse_args()

    lint_lines = read_file_lines(args.input_filename)

    handle_range(args.revision_range, lint_lines)


if __name__ == "__main__":
    main()



from __future__ import print_function, unicode_literals


def check_line_overlap_hunks(start, hunks, threshold):
    for change_from, change_to, _ in hunks:
        if check_line_in_range(start, change_to[0], change_to[1], threshold):
            return True
    return False


def check_line_in_range(line_number, range_start, range_count, threshold):
    in_range = range_start - threshold <= line_number <= range_start + range_count - 1 + threshold
    return in_range



from __future__ import print_function, unicode_literals

from collections import namedtuple
import re

LintEntry = namedtuple('LintEntry', ['filename', 'line', 'column', 'message'])


def parse_report(raw_report):
    pass


def parse_lint_line(line):
    """Parse lint diff line

    >>> parse_lint_line('foo.txt:1:2: bar')
    LintEntry(filename=u'foo.txt', start=1, count=2, content=u'bar')

    >>> parse_lint_line('foo.txt:123:50: bar')
    LintEntry(filename=u'foo.txt', start=123, count=50, content=u'bar')

    >>> parse_lint_line('foo.txt:0:1:')
    LintEntry(filename=u'foo.txt', start=0, count=1, content=u'')

    >>> parse_lint_line('foo/foo bar.txt:0:1: baz')
    LintEntry(filename=u'foo/foo bar.txt', start=0, count=1, content=u'baz')
    """
    # TODO: handle colon in filename
    line_parts = re.match('(?:\./)?(.+?):(\d+):(\d+): ?(.*)', line)
    lint_entry = LintEntry(
        line_parts.group(1),
        int(line_parts.group(2)),
        int(line_parts.group(3)),
        line_parts.group(4)
    )
    return lint_entry
